@@ -7,23 +7,23 @@ study_day = today
 
 import readchar  # 3rd party module for reading a character one at a time
 
-from . import cards
-col = cards.Collection()
+from vinca.lib import classes
+col = classes.Collection()
 col.cache_tags()
-decklist = cards.Decklist()
+deckdict = classes.Deckdict()
 
 from shutil import copytree, rmtree
 from sys import exit
 
-from . import editors, reviewers, schedulers
-from . import browser
+from vinca.lib import browser
+from vinca.lib.ansi import ansi
 
 vinca_path = Path(__file__) # /path/to/vinca/__init__.py
 vinca_path = vinca_path.parent # /path/to/vinca
-cards_path = vinca_path / 'cards'
-decks_path = vinca_path / 'decks.txt'
-decks_vimrc = vinca_path / 'decks.vimrc'
-tags_path = vinca_path / 'tags.txt'
+cards_path = vinca_path / 'data' / 'cards'
+decks_path = vinca_path / 'data' / 'decks.txt'
+decks_vimrc = vinca_path / 'lib' / 'decks.vimrc'
+tags_path = vinca_path / 'data' / 'tags.txt'
 
 
 def study(args):
@@ -32,6 +32,8 @@ def study(args):
 		queue += browser.filter(col, deck=deck, due_date_end=study_day)
 	for card in args.cards:
 		queue.append(card)
+	if not args.decks and not args.cards:
+		queue = browser.filter(col, due_date_end=study_day)
 	done_queue = []
 	while queue:
 		card = queue.pop()
@@ -50,11 +52,24 @@ def study(args):
 		if cont == 1:
 			card.schedule()
 
-def line_add(args):
+def linear_add(args):
+	new_card = classes.Card(create=True)
+	new_card.make_config('linear','linear','base')
+
+	tags_create = [tag for deck in args.decks for tag in deck.tags_create]
+	for tag in tags_create:
+		new_card.add_tag(tag)
+	
+	# TODO: timer
+	new_card.edit('a')
+	new_card.add_history(today, 0, 0)
+	new_card.schedule()
+	
+def one_liner(args):
 	tags_create = []
 	for deck in args.decks:
 		tags_create += deck.tags_create 
-	new_card = cards.Card(create=True)
+	new_card = classes.Card(create=True)
 	new_card.make_config('base','base','base')
 	for tag in tags_create:
 		new_card.add_tag(tag)
@@ -67,7 +82,7 @@ def add_basic(args):
 	tags_create = []
 	for deck in args.decks:
 		tags_create += deck.tags_create 
-	new_card = cards.Card(create=True)
+	new_card = classes.Card(create=True)
 	new_card.make_config('base','base','base')
 	for tag in tags_create:
 		new_card.add_tag(tag)
@@ -100,6 +115,7 @@ def statistics(args):
 		print(f'{len(due_cards)} due today')
 	for card in args.cards:
 		print(f'\nCard #{card.id}')
+		print(str(card)[:50])  # TODO
 		print(f'Due: {card.due_date}')
 		print(f'Tags: {" ".join(card.tags)}')
 		print(f'Date        Time   Grade')
@@ -115,120 +131,98 @@ def edit(args):
 			card.edit()
 		return
 	vim_cmd = ['vim',decks_path] 
-	vim_cmd += [f'+{args.decks[0].idx + 1}'] if args.decks else []
+	vim_cmd += [f'+{args.decks[0].id + 1}'] if args.decks else []
 	vim_cmd += ['-Nu',decks_vimrc]
 	vim_cmd += ['-c',f'set dictionary={tags_path}']
 	run(vim_cmd)
 def delete(args):
 	for deck in args.decks: 
-		decklist.delete(deck)
+		deckdict.delete(deck)
 	for card in args.cards:
+		if args.toggle_delete and card.deletedQ:
+			card.undelete()
+			continue
 		card.delete()
-arg_dict = {'a':add,
+cmd_dict = {'a':add,
 	    'A':add_many,
 	    'e':edit,
 	    'x':delete,
 	    's':study,
 	    'S':statistics,
-	    '1':line_add}
+	    'l':linear_add,
+	    '1':one_liner}
 		
-
-# deck manipulation
-def list_decks(args):
-	for i in range(len(decklist)):
-		deck = decklist[i]
-		print(deck.idx, deck, sep='\t')
-def visual_decks(args):
-	n = len(decklist)
-	selected = 0
-	run(['tput','civis'])
-	print('\n'*(n-1)) # move cursor down n lines
+def visual_select(args, iterable):
+	# allow for the selection of a card or deck
+	n = len(iterable)
+	sel = 0  # selected number
+	print(ansi['hide_cursor'],end='')
+	print('\n'*n,end='')  # move down n lines
+	line_width = 50  # TODO read from terminal
+	tab_width = 8
+	w = line_width - tab_width
 	while True:
-		for i in range(n):
-			run(['tput','cuu1']) # move up n lines
-		for i in range(n):
-			# print the decks with the selected deck highlighted
-			if i == selected:
-				run(['tput','smso'])
-			print(decklist[i])
-			run(['tput','sgr0'])
-		#get key
-		k = readchar.readchar()
-		if k == 'j' and selected < n - 1:
-			selected += 1
-		if k == 'k' and selected > 0:
-			selected -= 1
+		print(ansi['move_up_line']*n,end='') # move up n lines
+		for i, item in enumerate(iterable):
+			print(f"{ansi['reverse']*(i==sel)}{item.id}\t{str(item)[:w]}{ansi['reset']}")
+		k = readchar.readchar()  # get key
+		sel = (sel + (k=='j') - (k=='k') ) % n
 		if k == 'q' or k == readchar.key.ESC:
-			run(['tput','cvvis'])
+			print(ansi['show_cursor'],end='')
 			exit(0)
-			break
-		if k in arg_dict.keys():
-			# erase the menu
-			for i in range(n):
-				run(['tput','cuu1']) # move up n lines
-				run(['tput','el']) # and clear them as we go
-			# make cursor visible
-			run(['tput','cvvis'])
-			# select the selected deck
-			args.decks = [decklist[selected]]
-			# the key sets the same variable as that option normally would
-			cmd = arg_dict[k]
-			cmd(args)
-			break
+		if k in cmd_dict.keys():
+			if type(item) is classes.Deck: args.decks = [iterable[sel]]
+			if type(item) is classes.Card: args.cards = [iterable[sel]]
+			cmd_dict[k](args)
+			if k == 'S':
+				break
+			print(ansi['hide_cursor'],end='')
+
 def query(args):
-	matches = browser.search(col, args.pattern)
-	for card, hit in matches:
-		if args.index_only:
+	matches = browser.search(col, args.pattern)  # list of matching cards
+	for card in matches:
+		if args.id_only:
 			print(card.id)
-		else:
-			print(card.id, hit[:50], sep='\t')
+			continue
+		tabwidth = 8
+		line_width = 50  # TODO read from terminal
+		w = line_width - tabwidth
+		print(f'{card.id}\t{str(card)[:w]}')
 def visual_query(args):
 	matches = browser.search(col, args.pattern)
-	if not matches:
-		print('No matches.')
-		exit(0)
-	n = len(matches)
-	selected = 0
-	run(['tput','civis'])
-	print('\n'*(n-1)) # move down n lines
-	while True:
-		for i in range(n):
-			run(['tput','cuu1']) # move up n lines
-		for i,(card,hit) in enumerate(matches):
-			if i == selected:
-				run(['tput','smso']) # standout attribute
-			if card.deletedQ:
-				run(['tput','dim'])
-			print(f'{hit[:50]}')
-			run(['tput','sgr0']) # normal attribute
-		#get key
-		k = readchar.readchar()
-		if k == 'j' and selected < n - 1:
-			selected += 1
-		if k == 'k' and selected > 0:
-			selected -= 1
-		if k == 'q' or k == readchar.key.ESC:
-			run(['tput','cvvis'])
-			exit(0)
-			break
-		if k in arg_dict.keys():
-			# for i in range(n):
-			# 	run(['tput','cuu1']) # move up n lines
-			# 	run(['tput','el']) # and clear them as we go
-			#run(['tput','cvvis'])
-			args.cards = [matches[selected][0]]
-			cmd = arg_dict[k]
-			cmd(args)
-			run(['tput','civis'])
-			#break
+	args.toggle_delete = True  # allow for the 'x' key to restore cards
+	visual_select(args, matches)
+def list_decks(args):
+	assert deckdict, '''You have not created any decks!
+		Use [e] to create a deck.
+		Decks should be lines with the form:
+		name +tag_to_include -tag_to_exclude =tag_to_assign_to_new_cards'''
+	print(*[f'{d.id}\t{d}' for d in deckdict.list], sep='\n')
+def visual_decks(args):
+	assert deckdict, '''You have not created any decks!
+		Use [e] to create a deck.
+		Decks should be lines with the form:
+		name +tag_to_include -tag_to_exclude =tag_to_assign_to_new_cards'''
+	visual_select(args, deckdict.list)
+
 
 def purge(args):
 	for card in col:
 		if card.deletedQ:
-			rmtree(card.path)	
-# backup
+			rmtree(card.path)
+# backup / export
 def backup(args):
-	copytree(cards_path, args.small_backup)
+	backup_cards = []
+	if not args.cards and not args.decks:
+		backup_cards = col
+	for card in args.cards:
+		backup_cards.append(card)
+	for deck in args.decks:
+		backup_cards += browser.filter(col, deck=deck)
+
+	for card in backup_cards:
+		copytree(card.path, args.backup_dest / str(card.id))
+	
 def import_collection(args):
 	rmtree(cards_path)
 	copytree(args.import_path, cards_path)
